@@ -8,6 +8,8 @@ import tempfile
 import os
 from pkg_resources import resource_filename
 
+from .bgzf import BgzfBlocks
+
 # Misc
 MAX_RECENT_PROJECTS = 5
 MIN_COMPLETION_LETTERS = 1
@@ -35,6 +37,34 @@ CLASSIFICATION_ICONS = {
     5: 0xF03B1,
 }
 
+OPERATORS_PY_2_SQL = {
+    "$eq": "=",
+    "$gt": ">",
+    "$gte": ">=",
+    "$lt": "<",
+    "$lte": "<=",
+    "$in": "IN",
+    "$ne": "!=",
+    "$nin": "NOT IN",
+    "$regex": "REGEXP",
+    "$and": "AND",
+    "$or": "OR",
+}
+
+OPERATORS_SQL_2_PY = {
+    "=": "$eq",
+    ">": "$gt",
+    ">=": "$gte",
+    "<": "$lt",
+    "<=": "$lte",
+    "IN": "$in",
+    "!=": "$ne",
+    "NOT IN": "$nin",
+    "~": "$regexp",
+    "AND": "$and",
+    "OR": "$or",
+}
+
 # Genotypes
 GENOTYPE_ICONS = {-1: 0xF10D3, 0: 0xF0766, 1: 0xF0AA1, 2: 0xF0AA5}
 
@@ -69,7 +99,7 @@ WIKI_URL = "https://github.com/labsquare/cutevariant/wiki"
 
 # Logging
 LOGGER_NAME = "cutevariant"
-LOG_LEVEL = "DEBUG"
+LOG_LEVEL = "INFO"
 LOG_LEVELS = {
     "debug": logging.DEBUG,
     "info": logging.INFO,
@@ -143,20 +173,30 @@ def is_gz_file(filepath):
 
 
 def get_uncompressed_size(filepath):
-    """Get the size of the given compressed file
-    This size is stored in the last 4 bytes of the file.
-    """
-    with open(filepath, "rb") as file_obj:
-        file_obj.seek(-4, 2)
-        return int.from_bytes(file_obj.read(4), byteorder="little")
+    device = open(filepath, "rb")
+    magic_4bytes = device.read()[:4]
+    #  IT IS A GZIP FILE
+    if magic_4bytes == b"\x1f\x8b\x08\x08":
+        device.seek(-4, 2)
+        return int.from_bytes(device.read(4), byteorder="little")
+
+    #  IT IS A BGZIP FILE
+    elif magic_4bytes == b"\x1f\x8b\x08\x04":
+        device.seek(0)
+        return sum([i[3] for i in BgzfBlocks(device)])
+
+    else:
+        device = open(filepath, "rb")
+        device.seek(0, os.SEEK_END)
+        return device.tell()
 
 
 def bytes_to_readable(size) -> str:
     """return human readable size from bytes
-    
+
     Args:
         size (int): size in bytes
-    
+
     Returns:
         str: readable size
     """
@@ -203,5 +243,5 @@ def is_json_file(filename):
             json.load(file)
         except Exception as e:
             return False
-        finally:
-            return True
+
+    return True
