@@ -3,6 +3,10 @@ import io
 import gzip
 import math
 from collections import Counter
+
+# use to format value with URL caracter : #See Issue
+from urllib.parse import unquote
+
 import cutevariant.commons as cm
 
 LOGGER = cm.logger()
@@ -18,14 +22,13 @@ class AbstractReader(ABC):
         device: A file object typically returned by open(); Can be None if
             FakeReader type is instanciated.
         file_size: File size in bytes
-            See Also: :meth:`self.get_total_file_size`
         number_lines: Number of lines in the file (compressed or not).
             See Also: :meth:`self.compute_number_lines`
         read_bytes: Current bytes readed (progression = read_bytes / file_size)
             It's a fallback if number_lines can't be computed.
         samples: List of samples in the file (default: empty)
 
-        ignored_fields: Skip fields in importations. 
+        ignored_fields: Skip fields in importations.
             A list of fields to skip [{field_name:"AF", "category":"variant"}]
 
     Example:
@@ -40,8 +43,7 @@ class AbstractReader(ABC):
         self.read_bytes = 0
         self.samples = list()
 
-        self.file_size = self.get_total_file_size()
-        self.compute_number_lines()
+        self.file_size = 0
 
         self.ignored_fields = set()
 
@@ -256,19 +258,19 @@ class AbstractReader(ABC):
             duplicates.add(unique_key)
 
     def add_ignored_field(self, field_name: str, field_category: str):
-        """Add new field to the ignored_fields list. 
-        ignored fields will not returned by get_extra_fields and then are not imporpted 
-        into the database 
-        
+        """Add new field to the ignored_fields list.
+        ignored fields will not returned by get_extra_fields and then are not imporpted
+        into the database
+
         Args:
-            field_name (str): a field name 
+            field_name (str): a field name
             field_category (str): the category field name (variant,annotation,sample)
         """
 
         self.ignored_fields.add((field_name, field_category))
 
     def get_extra_variants(self, **kwargs):
-        """Yield variants with extra information computed.
+        """Yield variants with extra information computed and format if necessary
 
         The following information are added. See get_extra_fields
 
@@ -388,6 +390,29 @@ class AbstractReader(ABC):
                         if name in sample:
                             del sample[name]
 
+            # Format variant value ! For instance replace %3D by "="  using unquote
+            # See issue https://github.com/labsquare/cutevariant/issues/220
+
+            for key, value in variant.items():
+                if isinstance(value, str):
+                    variant[key] = unquote(value)
+
+            if "annotations" in variant:
+                for i, ann in enumerate(variant["annotations"]):
+                    for key, value in ann.items():
+                        if isinstance(value, str):
+                            variant["annotations"][i][key] = unquote(
+                                variant["annotations"][i][key]
+                            )
+
+            if "samples" in variant:
+                for i, sample in enumerate(variant["samples"]):
+                    for key, value in sample.items():
+                        if isinstance(value, str):
+                            variant["samples"][i][key] = unquote(
+                                variant["samples"][i][key]
+                            )
+
             yield nullify(variant)
 
     def get_extra_fields_by_category(self, category: str):
@@ -417,22 +442,6 @@ class AbstractReader(ABC):
         """
         return len(tuple(self.get_variants()))
 
-    def get_total_file_size(self) -> int:
-        """Compute file size int bytes"""
-        # FakeReader is used ?
-        if not self.device:
-            return 0
-
-        filename = self.device.name
-
-        if cm.is_gz_file(filename):
-            return cm.get_uncompressed_size(filename)
-        # Go to EOF and get position in bytes
-        size = self.device.seek(0, 2)
-        # Rewind the file
-        self.device.seek(0)
-        return size
-
     def compute_number_lines(self):
         """Get a sample of lines in file if possible and if the end of file is
         not reached compute an evaluation of the global number of lines.
@@ -515,14 +524,14 @@ def check_variant_schema(variant: dict):
                 {
                     "gene": str,
                     "transcript": str,
-                    Optional(str): Or(int, str, bool, float),
+                    Optional(str): Or(int, str, bool, float, None),
                 }
             ],
             Optional("samples"): [
                 {
                     "name": str,
                     "gt": And(int, lambda x: x in [-1, 0, 1, 2]),
-                    Optional(str): Or(int, str, bool, float),
+                    Optional(str): Or(int, str, bool, float, None),
                 }
             ],
         }
@@ -561,12 +570,13 @@ def check_field_schema(field: dict):
 
 def sanitize_field_name(field: str):
     # TODO
-    LOGGER.warning("NOT implemented function!!")
+    # LOGGER.warning("NOT implemented function!!")
+
     return field
 
 
 def nullify(variant: dict) -> dict:
-    """ Convert empty fields value  to NONE 
+    """Convert empty fields value  to NONE
     This is used have NULL value inside the SQLITE inside an empty string
 
     """
