@@ -32,17 +32,17 @@ import logging
 from pkg_resources import parse_version
 from functools import partial, lru_cache
 import itertools as it
+import numpy as np
 
 
 # Custom imports
 import cutevariant.commons as cm
+from cutevariant.core.querybuilder import build_sql_query
+from cutevariant.core.sql_aggregator import StdevFunc
 
 LOGGER = cm.logger()
 
 
-## Misc functions ==============================================================
-
-
 def get_sql_connection(filepath):
     """Open a SQLite database and return the connection object
 
@@ -53,6 +53,9 @@ def get_sql_connection(filepath):
         sqlite3.Connection: Sqlite3 Connection
             The connection is initialized with `row_factory = Row`.
             So all results are accessible via indexes or keys.
+            The connection also supports
+            - REGEXP function
+            - DESCRIBE_QUANT aggregate
     """
     connection = sqlite3.connect(filepath)
     # Activate Foreign keys
@@ -68,6 +71,7 @@ def get_sql_connection(filepath):
         return re.search(expr, str(item)) is not None
 
     connection.create_function("REGEXP", 2, regexp)
+    connection.create_aggregate("STD", 1, StdevFunc)
 
     if LOGGER.getEffectiveLevel() == logging.DEBUG:
         # Enable tracebacks from custom functions in DEBUG mode only
@@ -76,15 +80,19 @@ def get_sql_connection(filepath):
     return connection
 
 
+def get_database_file_name(conn):
+    return conn.execute("PRAGMA database_list").fetchone()["file"]
+
+
 def table_exists(conn: sqlite3.Connection, name: str) -> bool:
     """Return True if table exists
-    
+
     Args:
         conn (sqlite3.Connection): Sqlite3 connection
         name (str): Table name
-    
+
     Returns:
-        bool: True if table exists 
+        bool: True if table exists
     """
     c = conn.cursor()
     c.execute(f"SELECT name FROM sqlite_master WHERE name = '{name}'")
@@ -156,15 +164,81 @@ def create_indexes(conn: sqlite3.Connection):
 
 
 def count_query(conn, query):
-    """Count elements from the given query or table
-    """
+    """Count elements from the given query or table"""
     return conn.execute(f"SELECT COUNT(*) as count FROM ({query})").fetchone()[0]
 
 
+# Statistical data
+
+
+def get_stats_info(conn, field, source="variants", filters={}):
+    pass
+
+
+def get_field_info(conn, field, source="variants", filters={}, metrics=["mean", "std"]):
+    """
+    Returns statistical metrics for column field in conn
+    metrics is the list of statistical metrics you'd like to retrieve, among:
+    count,mean,std,min,q1,median,q3,max
+
+    For the metrics, you can also specify your own as a tuple by following the following format:
+
+    (metric_name,callable) where callable takes a numpy array and metric_name will be the key in the result
+    dictionnary. Example:
+
+    ("standard error",lambda array:np.std(array)/np.sqrt(len(array)))
+
+    The returned dict is in the form:
+    {
+        "mean" : 42,
+        "min" : 5,
+        "max" : 1000
+        "arbitrary_metric":15
+    }
+    It WILL and SHOULD change in the future
+    """
+
+    # metrics are literals, not column names, so add some single quotes to tell SQL
+
+    # TODO :
+
+    metric_functions = {
+        "count": len,
+        "mean": np.mean,
+        "std": np.std,
+        "min": lambda ar: np.quantile(ar, 0.0),
+        "q1": lambda ar: np.quantile(ar, 0.25),
+        "median": lambda ar: np.quantile(ar, 0.5),
+        "q3": lambda ar: np.quantile(ar, 0.75),
+        "max": lambda ar: np.quantile(ar, 1.0),
+    }
+
+    conn.row_factory = None
+    query = build_sql_query(conn, [field], source, filters, limit=None)
+
+    data = [i[0] for i in conn.execute(query)]
+
+    results = {}
+    for metric in metrics:
+        if metric in metric_functions:
+            value = metric_functions[metric](data)
+            results[metric] = value
+
+        if isinstance(metric, tuple) and len(metric) == 2:
+            metric_name, metric_func = metric
+            if callable(metric_func):
+                value = metric_func(data)
+                results[metric_name] = value
+
+    conn.row_factory = sqlite3.Row
+
+    return results
+
+
 ## project table ===============================================================
 
 
-def create_project(conn: sqlite3.Connection, name: str, reference: str):
+def create_table_project(conn: sqlite3.Connection, name: str, reference: str):
     """Create the table "projects" and insert project name and reference genome
 
     Args:
@@ -172,20 +246,27 @@ def create_project(conn: sqlite3.Connection, name: str, reference: str):
         name (str): Project name
         reference (str): Genom project
 
-    Todo:
-        * Rename to create_table_project
-
     """
-    cursor = conn.cursor()
-    cursor.execute(
-        "CREATE TABLE projects (id INTEGER PRIMARY KEY, name TEXT, reference TEXT)"
-    )
-    cursor.execute(
-        "INSERT INTO projects (name, reference) VALUES (?, ?)", (name, reference)
+    project_data = {"name": name, "reference": reference}
+
+    conn.execute("CREATE TABLE projects (id INTEGER PRIMARY KEY, key TEXT, value TEXT)")
+    conn.commit()
+
+    update_project(conn, project_data)
+
+
+def update_project(conn: sqlite3.Connection, project: dict):
+    conn.executemany(
+        "INSERT INTO projects (key, value) VALUES (?, ?)", list(project.items())
     )
     conn.commit()
 
 
+def get_project(conn: sqlite3.Connection):
+    g = (dict(data) for data in conn.execute("SELECT key, value FROM projects"))
+    return {data["key"]: data["value"] for data in g}
+
+
 ## metadatas table =============================================================
 
 
@@ -472,7 +553,7 @@ def create_selection_from_bed(
         source_query = f"""
         SELECT variants.id AS variant_id FROM variants
         INNER JOIN selections ON selections.name = '{source}'
-        INNER JOIN selection_has_variant AS sv ON sv.selection_id = selections.id AND sv.variant_id = variants.id 
+        INNER JOIN selection_has_variant AS sv ON sv.selection_id = selections.id AND sv.variant_id = variants.id
         """
 
     query = (
@@ -860,7 +941,7 @@ def get_field_range(conn, field_name: str, sample_name=None):
     return result
 
 
-def get_field_unique_values(conn, field_name: str, sample_name=None):
+def get_field_unique_values(conn, field_name: str, limit=None):
     """Return unique record values for a field name
 
     :param conn: sqlite3.connect
@@ -869,20 +950,31 @@ def get_field_unique_values(conn, field_name: str, sample_name=None):
     :return: list of unique values (can be empty if the field is not found)
     :rtype: list
     """
+
+    if field_name.startswith("ann."):
+        field_name = field_name.replace("ann.", "")
+
+    if field_name.startswith("samples."):
+        #  TODO replace samples ...
+        _, *_, field = field_name.split(".")
+        field_name = field
+
     field = get_field_by_name(conn, field_name)
     if not field:
         return []
     table = field["category"]  # variants, or annotations or samples
+
     if table == "samples":
-        if not sample_name:
-            raise ValueError("Pass sample parameter for sample fields")
-        query = f"""SELECT DISTINCT {field_name}
-        FROM sample_has_variant
-        JOIN samples ON sample_has_variant.sample_id = samples.id
-        WHERE samples.name='{sample_name}'
-        """
+        query = f""" SELECT DISTINCT `{field_name}` FROM sample_has_variant """
+
+    elif table == "annotations":
+        query = f""" SELECT DISTINCT `{field_name}` FROM annotations """
+
     else:
         query = f"SELECT DISTINCT `{field_name}` FROM {table}"
+
+    if limit:
+        query += " LIMIT " + str(limit)
     return [i[field_name] for i in conn.execute(query)]
 
 
@@ -1123,6 +1215,81 @@ def get_variants_count(conn):
     return count_query(conn, "variants")
 
 
+def get_variants(
+    conn: sqlite3.Connection,
+    fields,
+    source="variants",
+    filters={},
+    order_by=None,
+    order_desc=True,
+    limit=50,
+    offset=0,
+    group_by={},
+    having={},  # {"op":">", "value": 3  }
+    **kwargs,
+):
+
+    # TODO : rename as get_variant_as_tables ?
+
+    query = build_sql_query(
+        conn,
+        fields=fields,
+        source=source,
+        filters=filters,
+        order_by=order_by,
+        order_desc=order_desc,
+        limit=limit,
+        offset=offset,
+        group_by=group_by,
+        having=having,
+        **kwargs,
+    )
+
+    for i in conn.execute(query):
+        # THIS IS INSANE... SQLITE DOESNT RETURN ALIAS NAME WITH SQUARE BRACKET....
+        # I HAVE TO replace [] by () and go back after...
+        # TODO : Change VQL Syntax from [] to () would be a good alternative
+        # @See QUERYBUILDER
+        # See : https://stackoverflow.com/questions/41538952/issue-cursor-description-never-returns-square-bracket-in-column-name-python-2-7-sqlite3-alias
+        yield {k.replace("(", "[").replace(")", "]"): v for k, v in dict(i).items()}
+
+
+def get_variants_tree(
+    conn: sqlite3.Connection,
+    **kwargs,
+):
+    pass
+    # for variant in get_variants(conn, **kwargs):
+
+    #     item = {}
+    #     annotations = []
+    #     samples = {}
+
+    #     for key, value in variant.items():
+    #         if key.startswith("ann."):
+    #             value = str(value)
+    #             annotations.append(value[4:])
+
+    #         elif key.startswith("sample."):
+    #             _, *sample, field = key.split(".")
+    #             sample = ".".join(sample)
+
+    #             if "sample" not in samples:
+    #                 samples[sample] = list()
+    #             samples[sample].append()
+
+    #         else:
+    #             item[key] = value
+
+    #     if annotations:
+    #         item["annotations"] = annotations
+
+    #     if samples:
+    #         item["samples"] = samples
+
+    #     yield item
+
+
 def async_insert_many_variants(conn, data, total_variant_count=None, yield_every=3000):
     """Insert many variants from data into variants table
 
@@ -1205,6 +1372,7 @@ def async_insert_many_variants(conn, data, total_variant_count=None, yield_every
     # Loop over variants
     errors = 0
     progress = 0
+    variant_count = 0
     for variant_count, variant in enumerate(data, 1):
 
         # Insert current variant
@@ -1325,7 +1493,7 @@ def insert_many_variants(conn, data, **kwargs):
 ## samples table ===============================================================
 
 
-def create_table_samples(conn, fields=None):
+def create_table_samples(conn, fields=[]):
     """Create samples table
 
     :param conn: sqlite3.connect
@@ -1425,6 +1593,17 @@ def get_sample_annotations(conn, variant_id: int, sample_id: int):
     )
 
 
+def get_sample_annotations_by_variant(conn, variant_id: int):
+
+    conn.row_factory = sqlite3.Row
+    return [
+        dict(data)
+        for data in conn.execute(
+            f"SELECT * FROM sample_has_variant WHERE variant_id = {variant_id}"
+        )
+    ]
+
+
 def update_sample(conn, sample: dict):
     """Update sample record
 
