@@ -96,8 +96,6 @@ class VqlEditorWidget(plugin.PluginWidget):
             self.mainwindow.state.fields,
             self.mainwindow.state.source,
             self.mainwindow.state.filters,
-            self.mainwindow.state.group_by,
-            self.mainwindow.state.having,
         )
 
         self.set_vql(vql_query)
@@ -150,9 +148,14 @@ class VqlEditorWidget(plugin.PluginWidget):
             color = style.FIELD_TYPE.get(field["type"], "str")["color"]
             icon = FIcon(style.FIELD_TYPE.get(field["type"], "str")["icon"], "white")
 
-            if field["category"] == "variants" or field["category"] == "annotations":
+            if field["category"] == "variants":
                 self.text_edit.completer.model.add_item(name, description, icon, color)
 
+            if field["category"] == "annotations":
+                self.text_edit.completer.model.add_item(
+                    f"ann.{name}", description, icon, color
+                )
+
             if field["category"] == "samples":
                 # Overwrite name
                 for sample in samples:
@@ -228,8 +231,6 @@ class VqlEditorWidget(plugin.PluginWidget):
                 # name of the variant selection
                 self.mainwindow.state.source = cmd["source"]
                 self.mainwindow.state.filters = cmd["filters"]
-                self.mainwindow.state.group_by = cmd["group_by"]
-                self.mainwindow.state.having = cmd["having"]
                 # Refresh all plugins
                 self.mainwindow.refresh_plugins(sender=self)
                 continue
